using DslFactory.Utilities.DteCodeModel;
using EnvDTE;

namespace Dslfactory.wcfLanguage
{
 public partial class TypeOfInterest
 {
  #region FullName
  internal string GetFullNameValue()
  {
   if (string.IsNullOrEmpty(Namespace))
    return Name;
   else
    return Namespace + "." + Name;
  }

  internal void SetFullNameValue(string fullname)
  {
   // dont change store in undo
   if (Store.InUndoRedoOrRollback)
    return;

   if (string.IsNullOrEmpty(fullname))
   {
    Namespace = "";
    Name = "";
   }
   else
   {
    int index = fullname.LastIndexOf('.');
    if (index == -1)
    {
     Namespace = "";
     Name = fullname;
    }
    else
    {
     Namespace = fullname.Substring(0, index);
     Name = fullname.Substring(index + 1);
    }
   }
  }
  #endregion

  /// <summary>
  /// Get generation kind
  /// </summary>
  internal GenerationKind GetGenerationKindValue()
  {
   // Find known types
   KnownCodeTypes knownCodeTypes = KnownCodeTypes.FromStore(Store);
   if (knownCodeTypes == null)
    return GenerationKind.NotGenerated;

   // Find type
   CodeType codeType = knownCodeTypes.GetFullNamedType(FullName);
   if (codeType == null)
    return GenerationKind.NotGenerated;

   if (codeType.InfoLocation == vsCMInfoLocation.vsCMInfoLocationProject)
    return GenerationKind.GeneratedInProject;
   else
    return GenerationKind.GeneratedInReference;
  }


  /// <summary>
  /// Show the concepts used by this concept
  /// </summary>
  public abstract void ShowUsedConcepts();

  /// <summary>
  /// Get the Services definition this Typeof Interest belongs to
  /// </summary>
  /// <returns></returns>
  protected internal abstract ServicesDefinition GetServicesDefinition();
 }
}
